<?php

namespace Drupal\paragraphs_gantt\Plugin\Field\FieldFormatter;

use Drupal\Component\Serialization\Json;
use Drupal\Component\Utility\Html;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityReferenceSelection\SelectionPluginManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Field\FieldConfigInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\EntityReferenceFormatterBase;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Path\CurrentPathStack;
use Drupal\Core\Render\Markup;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Template\Attribute;
use Drupal\Core\Url;
use Drupal\field\Entity\FieldConfig;
use Drupal\paragraphs\Entity\Paragraph;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the paragraphs gantt formatter.
 *
 * @FieldFormatter(
 *   id = "paragraphs_gantt_formatter",
 *   module = "paragraphs_gantt",
 *   label = @Translation("Paragraphs gantt"),
 *   field_types = {
 *     "entity_reference_revisions"
 *   }
 * )
 */
class ParagraphsGanttFormatter extends EntityReferenceFormatterBase {

  /**
   * The fields of paragraphs.
   *
   * @var array
   */
  protected array $fieldsParagraphs;

  /**
   * Constructs a FormatterBase object.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\Core\Entity\EntityDisplayRepositoryInterface $entityDisplayRepository
   *   The entity display repository.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $dateFormatter
   *   The date formatter.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   *   The entity field manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity field manager.
   * @param \Drupal\Core\Path\CurrentPathStack $currentPathStack
   *   The current path stack.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The module handler.
   * @param \Drupal\Core\Entity\EntityReferenceSelection\SelectionPluginManagerInterface $selectionManager
   *   The entity reference selection plugin  manager.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The current user service.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $fileUrlGenerator
   *   File Url Generator service.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, protected EntityDisplayRepositoryInterface $entityDisplayRepository, protected DateFormatterInterface $dateFormatter, protected EntityFieldManagerInterface $entityFieldManager, protected EntityTypeManagerInterface $entityTypeManager, protected CurrentPathStack $currentPathStack, protected ModuleHandlerInterface $moduleHandler, protected SelectionPluginManagerInterface $selectionManager, protected AccountInterface $account, protected FileUrlGeneratorInterface $fileUrlGenerator) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('entity_display.repository'),
      $container->get('date.formatter'),
      $container->get('entity_field.manager'),
      $container->get('entity_type.manager'),
      $container->get('path.current'),
      $container->get('module_handler'),
      $container->get('plugin.manager.entity_reference_selection'),
      $container->get('current_user'),
      $container->get('file_url_generator'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    // Implement default settings.
    return [
      'view_mode' => 'default',
      'text' => 'field_gantt_text',
      'type' => 'field_gantt_type',
      'start_date' => 'field_gantt_date',
      'show_end' => FALSE,
      'duration' => 'field_gantt_duration',
      'progress' => 'field_gantt_progress',
      'open' => 'field_gantt_open',
      'parent' => 'field_gantt_parent',
      'links' => 'field_gantt_links',
      'owner' => 'field_gantt_owner',
      'custom_resource' => [],
      'planned_date' => 'field_gantt_planned_date',
      'split_actual_planned' => FALSE,
      'planned_duration' => 'field_gantt_planned_duration',
      'custom_field' => '',
      'toolbar_text' => '',
      'add_task' => TRUE,
      'edit_task' => TRUE,
      'native_dialog' => FALSE,
      'work_time' => FALSE,
      'work_day' => [0],
      'send_email' => FALSE,
      'send_notification' => FALSE,
      'cdn' => TRUE,
      'holidays' => '01-01,01-05',
      'creator' => 'field_owner_paragraph',
      'permission_edit' => FALSE,
      'show_button_detail' => FALSE,
      'show_column_resource' => [],
      'show_lightbox_resource' => [],
      'auto_schedule' => FALSE,
      'group_resource' => [],
      'select_parent' => FALSE,
      'resource_has_edit' => ['field_owner_paragraph'],
      'last_of_the_day' => TRUE,
      'dynamic_progress' => TRUE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $settingForm = [
      'view_mode' => [
        '#type' => 'select',
        '#options' => $this->entityDisplayRepository->getViewModeOptions($this->getFieldSetting('target_type')),
        '#title' => $this->t('View mode'),
        '#default_value' => $this->getSetting('view_mode'),
        '#required' => TRUE,
      ],
      'text' => [
        '#title' => $this->t('Title'),
        '#description' => $this->t('Field title'),
        '#type' => 'select',
        '#required' => TRUE,
        '#options' => $this->getConfigurableFields([
          'list_integer',
          'list_string',
          'text',
          'text_long',
          'text_with_summary',
          'string',
          'string_long',
        ]),
        '#empty_option' => $this->t('- None -'),
        '#default_value' => $this->getSetting('text'),
      ],
      'start_date' => [
        '#title' => $this->t('Date'),
        '#description' => $this->t('Link to Field date'),
        '#type' => 'select',
        '#required' => TRUE,
        '#options' => $this->getConfigurableFields(['datetime', 'daterange']),
        '#empty_option' => $this->t('- None -'),
        '#default_value' => $this->getSetting('start_date'),
      ],
      'show_end' => [
        '#type' => 'checkbox',
        '#title' => $this->t('Show end day'),
        '#default_value' => $this->getSetting('show_end'),
        '#description' => $this->t("If check it will add column end date."),
      ],
      'type' => [
        '#title' => $this->t('Type'),
        '#description' => $this->t('Link to Field type'),
        '#type' => 'select',
        '#required' => TRUE,
        '#options' => $this->getConfigurableFields(['list_string', 'string']),
        '#empty_option' => $this->t('- None -'),
        '#default_value' => $this->getSetting('type'),
      ],
      'duration' => [
        '#title' => $this->t('Duration'),
        '#description' => $this->t('Link to Field duration'),
        '#type' => 'select',
        '#empty_option' => $this->t('- None -'),
        '#options' => $this->getConfigurableFields(['integer']),
        '#default_value' => $this->getSetting('duration'),
      ],
      'work_time' => [
        '#type' => 'checkbox',
        '#title' => $this->t('Work day'),
        '#default_value' => $this->getSetting('work_time'),
        '#description' => $this->t("Enables calculating the duration of tasks in working time instead of calendar time."),
      ],
      'work_day' => [
        '#type' => 'select',
        '#title' => $this->t('Work day in week'),
        '#default_value' => $this->getSetting('work_day'),
        '#states' => [
          'visible' => [
            ':input[name="fields[' . $this->fieldDefinition->getName() . '][settings_edit_form][settings][work_time]"]' => ['checked' => TRUE],
          ], [
            ':input[name="style_options[work_time]"]' => ['checked' => TRUE],
          ],
        ],
        '#options' => [
          $this->t('Sunday'),
          $this->t('Monday'),
          $this->t('Tuesday'),
          $this->t('Wednesday'),
          $this->t('Thursday'),
          $this->t('Friday'),
          $this->t('Saturday'),
        ],
        '#description' => $this->t('Pick out non-working days of the week'),
        '#multiple' => TRUE,
      ],
      'holidays' => [
        '#title' => $this->t('Holidays'),
        '#description' => $this->t('Enter holidays separated by, Example "01-01,01-05,09-04-2023,... Format d-m or d-m-Y"'),
        '#type' => 'textarea',
        '#default_value' => $this->getSettings()['holidays'],
        '#states' => [
          'visible' =>
            [
              ':input[name="fields[' . $this->fieldDefinition->getName() . '][settings_edit_form][settings][work_time]"]' => ['checked' => TRUE],
            ],
          [
            ':input[name="style_options[work_time]"]' => ['checked' => TRUE],
          ],
        ],
      ],
      'progress' => [
        '#title' => $this->t('Progress'),
        '#description' => $this->t('Link to Field progress'),
        '#type' => 'select',
        '#empty_option' => $this->t('- None -'),
        '#options' => $this->getConfigurableFields([
          'float',
          'list_float',
          'decimal',
        ]),
        '#default_value' => $this->getSetting('progress'),
      ],
      'open' => [
        '#title' => $this->t('Open'),
        '#description' => $this->t('Link to Field open'),
        '#type' => 'select',
        '#empty_option' => $this->t('- None -'),
        '#options' => $this->getConfigurableFields(['boolean']),
        '#default_value' => $this->getSetting('open'),
      ],
      'parent' => [
        '#title' => $this->t('Parent'),
        '#description' => $this->t('Link to Field parent'),
        '#type' => 'select',
        '#empty_option' => $this->t('- None -'),
        '#options' => $this->getConfigurableFields([
          'integer',
          'list_integer',
        ]),
        '#default_value' => $this->getSetting('parent'),
      ],
      'links' => [
        '#title' => $this->t('Links type'),
        '#description' => $this->t('Link to Field link'),
        '#type' => 'select',
        '#empty_option' => $this->t('- None -'),
        '#options' => $this->getConfigurableFields([
          'double_field',
          'triples_field',
        ]),
        '#default_value' => $this->getSetting('links'),
      ],
      'owner' => [
        '#title' => $this->t('Resource'),
        '#description' => $this->t('Link to Field Resource Assignment'),
        '#type' => 'select',
        '#empty_option' => $this->t('- None -'),
        '#options' => $this->getConfigurableFields([
          'integer',
          'entity_reference',
        ]),
        '#default_value' => $this->getSetting('owner'),
      ],
      'custom_resource' => [
        '#title' => $this->t('Resource custom'),
        '#description' => $this->t('Link to Field Resource custom'),
        '#type' => 'select',
        '#empty_option' => $this->t('- None -'),
        '#options' => $this->getConfigurableFields([
          'entity_reference',
        ]),
        '#default_value' => $this->getSetting('custom_resource'),
        '#multiple' => TRUE,
      ],
      'show_column_resource' => [
        '#title' => $this->t('Column show resource'),
        '#description' => $this->t('Select the columns to be displayed'),
        '#type' => 'select',
        '#empty_option' => $this->t('- None -'),
        '#options' => $this->getConfigurableFields([
          'entity_reference',
        ]),
        '#default_value' => $this->getSetting('show_column_resource'),
        '#multiple' => TRUE,
      ],
      'show_lightbox_resource' => [
        '#title' => $this->t('Light box show resource'),
        '#description' => $this->t('Select the field to be displayed'),
        '#type' => 'select',
        '#empty_option' => $this->t('- None -'),
        '#options' => $this->getConfigurableFields([
          'entity_reference',
        ]),
        '#default_value' => $this->getSetting('show_lightbox_resource'),
        '#multiple' => TRUE,
      ],
      'group_resource' => [
        '#title' => $this->t('Group resource'),
        '#description' => $this->t('This function works on pro version'),
        '#type' => 'select',
        '#empty_option' => $this->t('- None -'),
        '#options' => $this->getConfigurableFields([
          'entity_reference',
        ]),
        '#default_value' => $this->getSetting('group_resource'),
        '#multiple' => TRUE,
      ],
      'auto_schedule' => [
        '#type' => 'checkbox',
        '#title' => $this->t('Auto schedule'),
        '#default_value' => $this->getSetting('auto_schedule'),
        '#description' => $this->t("Enables auto schedule tasks (This function only works on pro version)."),
      ],
      'creator' => [
        '#title' => $this->t('Author'),
        '#description' => $this->t('Link to Field author'),
        '#type' => 'select',
        '#empty_option' => $this->t('- None -'),
        '#options' => $this->getConfigurableFields(['entity_reference']),
        '#default_value' => $this->getSetting('creator'),
      ],
      'permission_edit' => [
        '#type' => 'checkbox',
        '#title' => $this->t('Permission to edit on a task'),
        '#default_value' => $this->getSetting('permission_edit'),
        '#description' => $this->t("Only author can edit their task."),
      ],
      'resource_has_edit' => [
        '#title' => $this->t('Resource has edit'),
        '#description' => $this->t('Resource with editing rights'),
        '#type' => 'select',
        '#empty_option' => $this->t('- None -'),
        '#options' => $this->getConfigurableFields([
          'entity_reference',
        ]),
        '#default_value' => $this->getSetting('resource_has_edit'),
        '#multiple' => TRUE,
        '#states' => [
          'visible' => [
            ':input[name="fields[' . $this->fieldDefinition->getName() . '][settings_edit_form][settings][permission_edit]"]' => ['checked' => TRUE],
          ],
        ],
      ],
      'add_task' => [
        '#type' => 'checkbox',
        '#title' => $this->t('Add Task'),
        '#default_value' => $this->getSetting('add_task'),
        '#description' => $this->t("Enable possibility 'Add Task' from Gantt chart."),
      ],
      'edit_task' => [
        '#type' => 'checkbox',
        '#title' => $this->t('Edit Task'),
        '#default_value' => $this->getSetting('edit_task'),
        '#description' => $this->t("Enable possibility 'Edit Task' from Gantt chart."),
      ],
      'show_button_detail' => [
        '#type' => 'checkbox',
        '#title' => $this->t('Show button details'),
        '#default_value' => $this->getSetting('show_button_detail'),
        '#description' => $this->t("Show details task with drupal form"),
      ],
      'send_email' => [
        '#type' => 'checkbox',
        '#title' => $this->t('Send an email after the change of status'),
        '#default_value' => $this->getSetting('send_email'),
      ],
      'cdn' => [
        '#title' => $this->t('Use CDN'),
        '#description' => $this->t('If not you can use custom version in libraries'),
        '#type' => 'checkbox',
        '#default_value' => $this->getSetting('cdn'),
      ],
      'last_of_the_day' => [
        '#title' => $this->t('Last of the day'),
        '#type' => 'checkbox',
        '#default_value' => $this->getSetting('last_of_the_day'),
      ],
      'dynamic_progress' => [
        '#title' => $this->t('Dynamic progress summary'),
        '#type' => 'checkbox',
        '#default_value' => $this->getSetting('dynamic_progress'),
        '#description' => $this->t('Recalculate progress of summary tasks when the progress of subtasks changes'),
      ],
      'custom_field' => [
        '#title' => $this->t('Custom field'),
        '#description' => $this->t('Additional column'),
        '#type' => 'select',
        '#empty_option' => $this->t('- None -'),
        '#options' => $this->getConfigurableFields(['integer']),
        '#default_value' => $this->getSetting('custom_field'),
      ],
      'toolbar_text' => [
        '#title' => $this->t('Custom text at the toolbar'),
        '#description' => $this->t('Variable available {{ paragraph_name }}, {{ paragraph_type }}, {{ entity_type }}, {{ entity_field }}, {{ entity_id }}'),
        '#type' => 'textarea',
        '#default_value' => $this->getSettings()['toolbar_text'],
      ],
      'baseline' => [
        '#type' => 'details',
        '#title' => $this->t('Base line (version PRO)'),
        '#description' => $this->t('If you want show double gantt to compare actual vs planned date. This method is available only in Pro versions'),

      ],
      'planned_date' => [
        '#type' => 'select',
        '#title' => $this->t('Planned Date field'),
        '#empty_option' => $this->t('- None -'),
        '#options' => $this->getConfigurableFields(['datetime', 'daterange']),
        '#default_value' => $this->getSetting('planned_date'),
        '#description' => $this->t('Select the field that contains the planned date range in the selected row. If you want show double gantt to compare actual vs planned date. This method is available only in Pro versions.'),
      ],
      'planned_duration' => [
        '#title' => $this->t('Duration'),
        '#description' => $this->t('Planned duration field'),
        '#type' => 'select',
        '#empty_option' => $this->t('- None -'),
        '#options' => $this->getConfigurableFields(['integer']),
        '#default_value' => $this->getSetting('planned_duration'),
        '#states' => [
          'visible' => [
            ':input[name="fields[' . $this->fieldDefinition->getName() . '][settings_edit_form][settings][planned_date]"]' => ['!value' => ''],
          ], [
            ':input[name="style_options[planned_date]"]' => ['!value' => ''],
          ],
        ],
      ],
      'split_actual_planned' => [
        '#type' => 'checkbox',
        '#title' => $this->t('Split gantt actual and planned date'),
        '#default_value' => $this->getSetting('split_actual_planned'),
        '#description' => $this->t("Uncheck to show date theory and actual date in the same baseline."),
        '#states' => [
          'visible' => [
            ':input[name="fields[' . $this->fieldDefinition->getName() . '][settings_edit_form][settings][planned_date]"]' => ['!value' => ''],
          ], [
            ':input[name="style_options[planned_date]"]' => ['!value' => ''],
          ],
        ],
      ],
      'select_parent' => [
        '#type' => 'checkbox',
        '#description' => $this->t("This selection allows to select the parent task in the lightbox."),
        '#title' => $this->t('Selection parent'),
        '#default_value' => $this->getSetting('select_parent'),
      ],
    ];

    if ($this->moduleHandler->moduleExists('pwa_firebase')) {
      $settingForm['send_notification'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Send notification to assign user'),
        '#default_value' => $this->getSetting('send_notification'),
      ];
    }
    if ($this->moduleHandler->moduleExists('paragraphs_table')) {
      $settingForm['native_dialog'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Custom form'),
        '#default_value' => $this->getSetting('native_dialog'),
        '#description' => $this->t("Use native Drupal form to add/edit."),
      ];
    }
    return $settingForm + parent::settingsForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    if (empty($this->fieldsParagraphs)) {
      $this->setFieldsParagraphs();
    }
    $summary = parent::settingsSummary();
    if (!empty($this->getSetting('text'))) {
      $summary[] = $this->t('Field title: @field', ['@field' => $this->getLabelName($this->getSetting('text'))]);
    }
    if (!empty($this->getSetting('start_date'))) {
      $summary[] = $this->t('Field date: @field', ['@field' => $this->getLabelName($this->getSetting('start_date'))]);
    }
    if (!empty($this->getSetting('duration'))) {
      $summary[] = $this->t('Field duration: @field', ['@field' => $this->getLabelName($this->getSetting('duration'))]);
    }
    if (!empty($this->getSetting('progress'))) {
      $summary[] = $this->t('Field progress: @field', ['@field' => $this->getLabelName($this->getSetting('progress'))]);
    }
    if (!empty($this->getSetting('open'))) {
      $summary[] = $this->t('Field open: @field', ['@field' => $this->getLabelName($this->getSetting('open'))]);
    }
    if (!empty($this->getSetting('parent'))) {
      $summary[] = $this->t('Field parent: @field', ['@field' => $this->getLabelName($this->getSetting('parent'))]);
    }
    if (!empty($this->getSetting('creator'))) {
      $summary[] = $this->t('Field owner create: @field', ['@field' => $this->getLabelName($this->getSetting('creator'))]);
    }
    if (!empty($this->getSetting('dynamic_progress'))) {
      $summary[] = $this->t('Dynamic progress summary: @field', ['@field' => $this->getLabelName($this->getSetting('dynamic_progress'))]);
    }
    return $summary;
  }

  /**
   * Get list of fields.
   */
  protected function getConfigurableFields($type = FALSE) {
    if (empty($this->fieldsParagraphs)) {
      $this->setFieldsParagraphs();
    }
    $listField = [];
    foreach ($this->fieldsParagraphs as $field_name => $field) {
      $listField[$field_name] = $field->getLabel();
      if (!empty($type)) {
        if (!is_array($type) && $type != $field->getType()) {
          unset($listField[$field_name]);
        }
        if (is_array($type) && !in_array($field->getType(), $type)) {
          unset($listField[$field_name]);
        }
      }
    }
    return $listField;
  }

  /**
   * {@inheritdoc}
   */
  protected function setFieldsParagraphs() {
    $targetType = $this->getFieldSetting('target_type');
    $targetBundle = array_key_first($this->fieldDefinition->getSetting("handler_settings")["target_bundles"]);
    /** @var \Drupal\paragraphs\ParagraphInterface $paragraphs_entity */
    $paragraphs_entity = $this->entityTypeManager->getStorage($targetType)
      ->create(['type' => $targetBundle]);
    $field_definitions = $paragraphs_entity->getFieldDefinitions();
    foreach ($field_definitions as $field_name => $field) {
      if ($field instanceof FieldConfigInterface || $field instanceof FieldConfig) {
        $this->fieldsParagraphs[$field_name] = $field;
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function getLabelName($fieldName) {
    if (!empty($this->fieldsParagraphs[$fieldName])) {
      return $this->fieldsParagraphs[$fieldName]->getLabel();
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {

    $setting = $this->getSettings();
    $targetType = $this->getFieldSetting('target_type');
    $field_definition = $items->getFieldDefinition();
    $entity_type_id = $field_definition->get('entity_type');
    $selectionHandler = $this->selectionManager->getSelectionHandler($field_definition);
    $bundles = $selectionHandler->entityTypeBundleInfo->getBundleInfo($targetType);
    $field_name = $field_definition->getName();
    $field_label = $field_definition->getLabel();
    $entity = $items->getEntity();
    $entityId = $entity->id();
    $currentURL = $this->currentPathStack->getPath();
    $destination = ['destination' => $currentURL];
    $view_mode = $this->viewMode == 'full' ? 'default' : $this->viewMode;
    $dialog_width = '80%';
    $handlers = $field_definition->getSetting("handler_settings");
    $handler = array_key_first($handlers['target_bundles']);
    $storage = $this->entityTypeManager->getStorage('paragraphs_type');
    $paragraphs_type = $storage->load($handler);

    // Get list owner.
    $owner = [];
    if (!empty($setting['owner'])) {
      $paragraph = Paragraph::create(['type' => $handler]);
      $fieldDefinitions = $paragraph->getFieldDefinitions();
      $listOwners = $this->selectionManager
        ->getSelectionHandler($fieldDefinitions[$setting['owner']], $paragraph)
        ->getReferenceableEntities();
      if (!empty($listOwners['user'])) {
        $users = $listOwners['user'];
        if (!empty($users[0])) {
          unset($users[0]);
        }
        $colorIndex = 0;
        foreach ($users as $uid => $userName) {
          $owner[] = [
            'key' => $uid,
            'label' => $userName,
            'textColor' => "#FFF",
            'backgroundColor' => $this->generateColor($colorIndex++),
          ];
        }
      }
    }

    // Get list custom resource.
    if (!empty($setting['custom_resource'])) {
      $listSource = [];
      $list_source = [];
      $sourceBundleMapping = [];
      $setting_custom_resource = [];
      $group_resource = [];
      if ($setting['group_resource']) {
        $group_resource[] = [
          "name" => $this->t("Default"),
          "field_resource" => $this->t("default"),
        ];
      };
      $paragraph = Paragraph::create(['type' => $handler]);
      $fieldDefinitions = $paragraph->getFieldDefinitions();
      foreach ($setting['custom_resource'] as $field) {
        $sourceBundleMapping[$field] = $fieldDefinitions[$field]->getSetting('handler_settings')['target_bundles'] ?? ["user" => "user"];
        if ($field != $setting['owner']) {
          if (in_array($field, $setting['group_resource'])) {
            $group_resource[] = [
              "name" => $fieldDefinitions[$field]->getLabel(),
              "field_resource" => $field,
            ];
          }
          $setting_custom_resource[] = $field;
          $listSource[$field] = $this->selectionManager
            ->getSelectionHandler($fieldDefinitions[$field], $paragraph)
            ->getReferenceableEntities();
        }
      }

      if (!empty($sourceBundleMapping)) {
        foreach ($sourceBundleMapping as $field => $bundleLinks) {
          $list_source[$field] = [];
          if (!empty($bundleLinks)) {
            foreach ($bundleLinks as $bundleName) {
              if (!empty($listSource[$field][$bundleName])) {
                foreach ($listSource[$field][$bundleName] as $id => $name) {
                  $list_source[$field][] = [
                    'key' => $id,
                    'label' => $name,
                  ];
                }
              }
            }
          }
        }
      }
    }

    // Custom field.
    if (!empty($setting['custom_field']) && !empty($fieldDefinitions[$setting['custom_field']])) {
      $setting['custom_field_name'] = $fieldDefinitions[$setting['custom_field']]->getLabel();
    }
    // Label resource.
    $resource_label = [];
    if (!empty($setting['custom_resource'])) {
      foreach ($setting['custom_resource'] as $resource) {
        if (!empty($fieldDefinitions[$resource])) {
          $resource_label[$resource] = $fieldDefinitions[$resource]->getLabel();
        }
      }
    }

    // Format date.
    $dateType = 'date';
    $dateFormatStorage = $this->entityTypeManager->getStorage('date_format');
    $drupalShortFormat = $dateFormatStorage->load('short')->getPattern();
    $patternDate = explode(' ', $drupalShortFormat);
    $setting['format_date'] = current($patternDate);
    $paragraph = Paragraph::create(['type' => $paragraphs_type->id()]);
    $fieldDefinitions = $paragraph->getFieldDefinitions();
    if (!empty($fieldDefinitions[$setting['start_date']])) {
      $dateSettings = $fieldDefinitions[$setting['start_date']]->getSettings();
      $dateType = $dateSettings["datetime_type"];
    }
    if ($dateType == 'date') {
      $setting['format_date'] = current(explode(' ', $setting['format_date']));
    }

    $path = '';
    if (method_exists($paragraphs_type, 'getIconUrl')) {
      $path = $paragraphs_type->getIconUrl();
    }
    $title = implode(' ', [
      '<i class="bi bi-calendar-plus"></i>',
      $this->t('Add'),
      $bundles[$handler]['label'],
    ]);
    if (!empty($path)) {
      $title = '<img src="' . $path . '" height="25"/> ' . $title;
      $field_label = '<img src="' . $path . '" height="25"/> ' . $field_label;
    }
    $ganttId = $field_name . '_' . $handler . '_' . $entityId;
    $attr = [
      'class' => ['gantt', $handler],
      'id' => Html::getId($ganttId),
    ];
    $lib = 'paragraphs_gantt/paragraphs-gantt';
    if ($setting['cdn']) {
      $lib .= '.cdn';
    }
    $data = $this->convertGanttData($items, $langcode, $setting, $destination);
    $context = [
      'paragraph_type' => $handler,
      'entity_type' => $entity_type_id,
      'entity_field' => $field_name,
      'entity_id' => $entityId,
      'paragraph_name' => $bundles[$handler]['label'],
    ];
    $add_link = Url::fromRoute('paragraphs_gantt.add', $context)
      ->setOptions(['query' => $destination]);
    $ajax_url = Url::fromRoute('paragraphs.gantt', [
      'paragraph_type' => $handler,
      'view_mode' => $view_mode,
      'entity_type' => $entity_type_id,
      'entity_field' => $field_name,
      'entity_id' => $entityId,
    ]);
    // Convert holiday to format Y-m-d.
    if (!empty($setting['holidays'])) {
      $current_year = date("Y");
      $new_date_array = [];
      $holidays = explode(',', str_replace([' ', PHP_EOL], [
        '',
        ',',
      ], $setting['holidays']));
      foreach ($holidays as $holiday) {
        $extract = explode('-', $holiday);
        $day = $extract[0];
        $month = $extract[1];
        $year = $extract[2] ?? $current_year;
        $new_date_array[] = date('Y-m-d', strtotime("$year-$month-$day"));
      }
      $setting['holidays'] = $new_date_array;
    }
    // Get patch css.
    $module_path = $this->moduleHandler->getModule('paragraphs_gantt')->getPath();
    $arrSource = [
      $module_path,
      'css',
      'reset.css',
    ];
    $link_css = implode(DIRECTORY_SEPARATOR, $arrSource);
    $link_css = $this->fileUrlGenerator->generateAbsoluteString($link_css);
    $elements = [
      '#toolbar_text' => [
        '#type' => 'inline_template',
        '#template' => $setting['toolbar_text'],
        '#context' => $context,
      ],
      '#type' => 'container',
      '#theme' => 'paragraphs_gantt_wrapper',
      '#field_name' => $field_name,
      '#title' => $field_label,
      '#id' => $ganttId,
      '#id_gantt' => $ganttId,
      '#owner' => $owner,
      '#toolbar_option' => [
        'auto_schedule' => $setting['auto_schedule'],
        'group_resource' => $group_resource ?? [],
      ],
      '#resource' => $list_source ?? [],
      '#data' => $data,
      "#paragraph_parent_id" => $entityId,
      "#paragraph_parent_type" => $entity_type_id,
      "#paragraph_parent_field_name" => $field_name,
      '#attributes' => new Attribute($attr),
      '#ajax' => $ajax_url,
      '#add' => [
        '#type' => 'link',
        '#url' => $add_link,
        '#title' => Markup::create($title),
        '#attributes' => [
          'class' => ['use-ajax', 'btn', 'btn-default'],
          'data-dialog-type' => 'modal',
          'data-dialog-options' => Json::encode(['width' => $dialog_width]),
        ],
      ],
      '#settings' => $setting,
      '#attached' => [
        'drupalSettings' => [
          'link_css' => $link_css,
          'current_user' => $this->account->id(),
          'date_format' => str_replace(
            ['Y', 'm', 'd', 'H', 'i', 's'],
            ['%Y', '%m', '%d', '%H', '%i', '%s'],
            $setting['format_date']
          ),
          'gantt' => [
            $ganttId => [
              'id' => 'gantt_' . $ganttId,
              'owner' => $setting['owner'],
              'custom_resource' => !empty($setting_custom_resource) ? $setting_custom_resource : [],
              'permission_edit' => $setting['permission_edit'],
              'show_button_detail' => $setting['show_button_detail'],
              'split_actual_planned' => !empty($setting["split_actual_planned"]),
              'add_task' => !empty($setting["add_task"]),
              'add_link' => $add_link->toString(),
              'parent_field' => $setting['parent'] ?? FALSE,
              'edit_task' => !empty($setting["edit_task"]),
              'native_dialog' => !empty($setting["native_dialog"]),
              'work_time' => !empty($setting['work_time']),
              'work_day' => $setting['work_day'],
              'holidays' => $setting['holidays'],
              'setting_resource' => [
                'resource_label' => $resource_label ? $resource_label : [],
                'resource_column' => $setting['show_column_resource'] ? $setting['show_column_resource'] : [],
                'resource_lightbox' => $setting['show_lightbox_resource'] ? $setting['show_lightbox_resource'] : [],
              ],
              'has_duration' => !empty($setting['duration']),
              'has_planned' => !empty($data['planned']),
              'show_end' => !empty($setting['show_end']),
              'custom_field' => $setting['custom_field'] ?? FALSE,
              'custom_field_name' => $setting['custom_field_name'] ?? FALSE,
              'current_path' => 'destination=' . $this->currentPathStack->getPath(),
              'ajax' => $ajax_url->toString(),
              'use_cdn' => $setting['cdn'],
              'select_parent' => $setting['select_parent'],
              'auto_schedule' => $setting['auto_schedule'],
              'last_of_the_day' => $setting['last_of_the_day'],
              'dynamic_progress' => $setting['dynamic_progress'],
              'link_detail' => [
                'display' => Url::fromRoute('paragraphs_gantt.display', ['paragraph' => "-arg_0-"])
                  ->setOptions(['query' => $destination])
                  ->toString(),
                'edit' => Url::fromRoute('paragraphs_gantt.edit', ['paragraph' => "-arg_0-"])
                  ->setOptions(['query' => $destination])
                  ->toString(),
              ],
              'import' => Url::fromRoute('paragraphs.import', [
                'paragraph_type' => $handler,
                'view_mode' => $view_mode,
                'entity_type' => $entity_type_id,
                'entity_field' => $field_name,
                'entity_id' => $entityId,
              ])->toString(),
            ],
          ],
        ],
        'library' => [$lib],
      ],
    ];
    if ($setting['owner'] || !empty($setting['custom_resource'])) {
      $elements['#attached']['library'][] = 'paragraphs_gantt/paragraphs-gantt.chosen';
    }
    if ($setting['split_actual_planned']) {
      $elements["#dataPlanned"] = !empty($data['planned']) ? ['data' => $data['planned']] : FALSE;
      if (!empty($data['links'])) {
        if (!empty($elements["#dataPlanned"])) {
          $elements["#dataPlanned"]['links'] = $data['links'];
        }
      }
    }

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  protected function convertGanttData(FieldItemListInterface $items, string $langcode, array $setting, $destination = NULL) {

    $entities = array_values($this->getEntitiesToView($items, $langcode));
    // When a display renderer doesn't exist, fall back to the default.
    $field_definition = $items->getFieldDefinition();
    $handlers = $field_definition->getSetting("handler_settings");
    $targetBundle = array_key_first($handlers['target_bundles']);
    $links = [];
    $outData = [];
    $mappingGantt = [
      'text',
      'type',
      'start_date',
      'duration',
      'progress',
      'open',
      'parent',
      'owner',
      'planned_date',
      'planned_duration',
      'custom_field',
      'creator',
      'custom_resource',
      'resource_has_edit',
    ];
    $dataPlanned = [];
    foreach ($entities as $delta => $entity) {
      if ($targetBundle != $entity->bundle()) {
        continue;
      }

      $data[$delta] = ['id' => $entity->id()];
      foreach ($mappingGantt as $ganttField) {
        if (!is_array($setting[$ganttField]) && !empty($field_name = $setting[$ganttField]) && !empty($entity->$field_name)) {
          $value = $entity->get($field_name)->getString();
          if (!empty($value)) {
            $data[$delta][$ganttField] = $value;
          }
          if ($ganttField == 'start_date' && !$entity->get($field_name)->isEmpty()) {
            $data[$delta][$ganttField] = $entity->get($field_name)->value;
          }
          if ($ganttField == 'end_date' && !$entity->get($field_name)->isEmpty()) {
            $data[$delta][$ganttField] = $entity->get($field_name)->end_value;
          }

          // Process creator.
          if ($ganttField == 'creator') {
            $data[$delta][$ganttField] = [];
            $creators = !$entity->get($field_name)
              ->isEmpty() ? $entity->get($field_name)->getValue() : [];
            if ($creators) {
              foreach ($creators as $index => $value) {
                $creator = $entity->get($field_name)->referencedEntities();
                $data[$delta][$ganttField][] = $value["target_id"];
                $data[$delta]["creator_name"][] = !empty($creator) ? $creator[$index]->getDisplayName() : $this->t('User deleted');
              }
            }
          }

          // Process planned date.
          if ($ganttField == 'planned_date' && !$entity->get($field_name)->isEmpty()) {
            if (!empty($entity->get($field_name)->value)) {
              $dateValueStart = new DrupalDateTime($entity->get($field_name)->value);
              $data[$delta]['planned_start'] = $dateValueStart->format("Y-m-d");
              $dateValueEnd = new DrupalDateTime($entity->get($field_name)->end_value);
              $data[$delta]['planned_end'] = $dateValueEnd->format("Y-m-d");
              // Option Last of the day.
              if ($setting['last_of_the_day']) {
                $dateValueEnd->modify('+1 day');
                $data[$delta]['planned_end'] = $dateValueEnd->format("Y-m-d");
              }
              if (!empty($data[$delta]['planned_start'])) {
                $dataPlanned[$delta] = $data[$delta];
              }
            }
          }

          // Process owner.
          if ($ganttField == 'owner') {
            $data[$delta][$ganttField] = [];
            if (!$entity->get($field_name)->isEmpty()) {
              $values = $entity->get($field_name)->getValue();
              foreach ($values as $value) {
                if (!empty($value->entity)) {
                  $data[$delta][$ganttField][] = $value->entity->id();
                }
                elseif (is_array($value) && !empty($value['target_id'])) {
                  $data[$delta][$ganttField][] = $value['target_id'];
                }
              }
            }
          }

          // Process duration.
          if ($ganttField == 'duration') {
            $data[$delta][$ganttField] = !empty($value) ? $value : 1;
          }

          // Process custom field.
          if ($ganttField == 'custom_field') {
            $data[$delta][$field_name] = $value;
          }
        }

        // Process resource.
        if ($ganttField == 'custom_resource') {
          foreach ($setting[$ganttField] as $item) {
            $data[$delta][$item] = [];
            if (!$entity->get($item)->isEmpty()) {
              $values = $entity->get($item)->getValue();
              foreach ($values as $value) {
                if (!empty($value->entity)) {
                  $data[$delta][$item][] = $value->entity->id();
                }
                elseif (is_array($value) && !empty($value['target_id'])) {
                  $data[$delta][$item][] = $value['target_id'];
                }
              }
            }
          }
        }

        // Link detail of task.
        if ($setting['show_button_detail']) {
          $data[$delta]["link_detail"] = Url::fromRoute('paragraphs_gantt.edit', ['paragraph' => $entity->id()])
            ->setOptions(['query' => $destination])->toString();
        }

        // Resource edit.
        if (!empty($setting['permission_edit']) && $ganttField == 'resource_has_edit' && !empty($setting[$ganttField])) {
          $data[$delta]['has_edit'] = [];
          foreach ($setting[$ganttField] as $item) {
            if (!$entity->get($item)->isEmpty()) {
              $values = $entity->get($item)->getValue();
              foreach ($values as $value) {
                if (!empty($value->entity)) {
                  $data[$delta]['has_edit'][] = $value->entity->id();
                }
                elseif (is_array($value) && !empty($value['target_id'])) {
                  $data[$delta]['has_edit'][] = $value['target_id'];
                }
              }
            }
          }
          if (!in_array($this->account->id(), $data[$delta]['has_edit'])) {
            $data[$delta]['readonly'] = TRUE;
            $data[$delta]["link_detail"] = Url::fromRoute('paragraphs_gantt.display', ['paragraph' => $entity->id()])
              ->setOptions(['query' => $destination])->toString();
          }
        }
      }

      // Process Links.
      if (!empty($setting['links'])) {
        $valueLinks = $entity->get($setting['links'])->getValue();
        foreach ($valueLinks as $valueLink) {
          $target_entity = NULL;
          if (!empty($valueLink['first'])) {
            $target_entity = Paragraph::load($valueLink['first']);
          }
          if (!empty($setting['permission_edit']) && $ganttField == 'resource_has_edit' && !empty($setting[$ganttField]) && !empty($target_entity)) {
            $has_edit_links = [];
            foreach ($setting[$ganttField] as $item) {
              if (!$target_entity->get($item)->isEmpty()) {
                $values = $target_entity->get($item)->getValue();
                foreach ($values as $value) {
                  if (!empty($value->entity)) {
                    $has_edit_links[] = $value->entity->id();
                  }
                  elseif (is_array($value) && !empty($value['target_id'])) {
                    $has_edit_links[] = $value['target_id'];
                  }
                }
              }
            }
            if (!in_array($this->account->id(), $has_edit_links)) {
              $links[] = [
                'id' => implode('-', [$entity->id(), $valueLink['first']]),
                'source' => (int) $entity->id(),
                'target' => (int) $valueLink['first'],
                'type' => $valueLink['second'],
                'lag' => $valueLink['third'] ?? NULL,
                'readonly' => TRUE,
              ];
            }
            else {
              $links[] = [
                'id' => implode('-', [$entity->id(), $valueLink['first']]),
                'source' => (int) $entity->id(),
                'target' => (int) $valueLink['first'],
                'type' => $valueLink['second'],
                'lag' => $valueLink['third'] ?? NULL,
                'readonly' => FALSE,
              ];
            }
          }
          else {
            $links[] = [
              'id' => implode('-', [$entity->id(), $valueLink['first']]),
              'source' => (int) $entity->id(),
              'target' => (int) $valueLink['first'],
              'type' => $valueLink['second'],
              'lag' => $valueLink['third'] ?? NULL,
              'readonly' => TRUE,
            ];
          }
        }
      }
    }
    // Data planned.
    if (!empty($dataPlanned)) {
      foreach ($dataPlanned as $planned) {
        $planned['start_date'] = $planned['planned_start'];
        $planned['duration'] = $this->getDuration($planned);
        $outData['planned'][] = $planned;
      }
    }
    if (!empty($data)) {
      $outData['data'] = $data;
      if (!empty($links)) {
        $outData['links'] = $links;
      }
    }
    return $outData;
  }

  /**
   * {@inheritDoc}
   */
  protected function getDuration($planned = []) {
    if ($planned['planned_end'] == $planned['planned_start']) {
      return 1;
    }
    if (!empty($planned['planned_date'])) {
      [$start, $end] = explode(', ', $planned['planned_date']);
    }
    else {
      $start = str_replace('/', '-', $planned['planned_start']);
      $end = str_replace('/', '-', $planned['planned_end']);
    }

    return (int) abs(strtotime($end) - strtotime($start)) / 86400;
  }

  /**
   * Change color backgroup foreach user.
   *
   * @param int $index
   *   Index user in array.
   *
   * @return string
   *   Code color.
   */
  private function generateColor(int $index) {
    $listColor = [
      "#03A9F4",
      "#f57730",
      "#e157de",
      "#78909C",
      "#8D6E63",
      "#800000",
      "#008080",
      "#d2691e",
      "#ff8c00",
      "#daa520",
      "#4b0082",
      "#FF0000",
      "#800080",
      "#008000",
      "#808000",
      "#000080",
      "#0000FF",
    ];
    if (empty($listColor[$index])) {
      return $listColor[$index % count($listColor)];
    }
    return $listColor[$index];
  }

}
